<?php

namespace Drupal\auto;

use Drupal\Core\Entity\EntityAccessControlHandler;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Access\AccessResult;

/**
 * Access controller for the Auto entity.
 *
 * @see \Drupal\auto\Entity\Auto.
 */
class AutoAccessControlHandler extends EntityAccessControlHandler {

  /**
   * {@inheritdoc}
   */
  protected function checkAccess(EntityInterface $entity, $operation, AccountInterface $account) {
    /** @var \Drupal\auto\Entity\AutoInterface $entity */
    switch ($operation) {
      case 'view':
        if (!$entity->isPublished()) {
          return AccessResult::allowedIfHasPermission($account, 'view unpublished auto entities');
        }
        return AccessResult::allowedIfHasPermission($account, 'view published auto entities');

      case 'update':
        return AccessResult::allowedIfHasPermission($account, 'edit auto entities');

      case 'delete':
        return AccessResult::allowedIfHasPermission($account, 'delete auto entities');
    }

    // Unknown operation, no opinion.
    return AccessResult::neutral();
  }

  /**
   * {@inheritdoc}
   */
  protected function checkCreateAccess(AccountInterface $account, array $context, $entity_bundle = NULL) {
    return AccessResult::allowedIfHasPermission($account, 'add auto entities');
  }

}
