<?php

namespace Drupal\auto\Entity;

use Drupal\Core\Entity\EntityStorageInterface;
use Drupal\Core\Field\BaseFieldDefinition;
use Drupal\Core\Entity\EntityChangedTrait;
use Drupal\Core\Entity\EntityTypeInterface;
use Drupal\user\UserInterface;

/**
 * Defines the Volante entity.
 *
 * @ingroup auto
 *
 * @ContentEntityType(
 *   id = "volante",
 *   label = @Translation("Volante"),
 *   handlers = {
 *     "view_builder" = "Drupal\Core\Entity\EntityViewBuilder",
 *     "list_builder" = "Drupal\auto\VolanteListBuilder",
 *     "views_data" = "Drupal\auto\Entity\VolanteViewsData",
 *
 *     "form" = {
 *       "default" = "Drupal\auto\Form\VolanteForm",
 *       "add" = "Drupal\auto\Form\VolanteForm",
 *       "edit" = "Drupal\auto\Form\VolanteForm",
 *       "delete" = "Drupal\auto\Form\VolanteDeleteForm",
 *     },
 *     "access" = "Drupal\auto\VolanteAccessControlHandler",
 *     "route_provider" = {
 *       "html" = "Drupal\auto\VolanteHtmlRouteProvider",
 *     },
 *   },
 *   base_table = "volante",
 *   admin_permission = "administer volante entities",
 *   entity_keys = {
 *     "id" = "id",
 *     "label" = "rivestimento",
 *     "uuid" = "uuid",
 *     "uid" = "user_id",
 *     "langcode" = "langcode",
 *     "status" = "status",
 *   },
 *   links = {
 *     "canonical" = "/auto/volante/{volante}",
 *     "add-form" = "/auto/volante/add",
 *     "edit-form" = "/auto/volante/{volante}/edit",
 *     "delete-form" = "/auto/volante/{volante}/delete",
 *     "collection" = "/auto/volante",
 *   },
 *   field_ui_base_route = "volante.settings"
 * )
 * 
 * @property \Drupal\auto\Adapter\VolanteAdapter $adapter
 */
class Volante extends ContentEntityExtended implements VolanteInterface {

  use EntityChangedTrait;
  
  /**
   * {@inheritdoc}
   */
  protected function adapterClass(): string {
    return '\Drupal\auto\Adapter\VolanteAdapter';
  }

  /**
   * {@inheritdoc}
   */
  public static function preCreate(EntityStorageInterface $storage_controller, array &$values) {
    parent::preCreate($storage_controller, $values);
    $values += [
      'user_id' => \Drupal::currentUser()->id(),
    ];
  }

  /**
   * {@inheritdoc}
   */
  public function getName() {
    return $this->get('name')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setName($name) {
    $this->set('name', $name);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getCreatedTime() {
    return $this->get('created')->value;
  }

  /**
   * {@inheritdoc}
   */
  public function setCreatedTime($timestamp) {
    $this->set('created', $timestamp);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function getOwner() {
    return $this->get('user_id')->entity;
  }

  /**
   * {@inheritdoc}
   */
  public function getOwnerId() {
    return $this->get('user_id')->target_id;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwnerId($uid) {
    $this->set('user_id', $uid);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function setOwner(UserInterface $account) {
    $this->set('user_id', $account->id());
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public function isPublished() {
    return (bool) $this->getEntityKey('status');
  }

  /**
   * {@inheritdoc}
   */
  public function setPublished($published) {
    $this->set('status', $published ? TRUE : FALSE);
    return $this;
  }

  /**
   * {@inheritdoc}
   */
  public static function baseFieldDefinitions(EntityTypeInterface $entity_type) {
    $fields = parent::baseFieldDefinitions($entity_type);

    $fields['user_id'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Authored by'))
      ->setDescription(t('The user ID of author of the Volante entity.'))
      ->setRevisionable(TRUE)
      ->setSetting('target_type', 'user')
      ->setSetting('handler', 'default')
      ->setTranslatable(TRUE)
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'type' => 'author',
        'weight' => 0,
      ])
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'weight' => 5,
        'settings' => [
          'match_operator' => 'CONTAINS',
          'size' => '60',
          'autocomplete_type' => 'tags',
          'placeholder' => '',
        ],
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

//    $fields['name'] = BaseFieldDefinition::create('string')
//      ->setLabel(t('Name'))
//      ->setDescription(t('The name of the Volante entity.'))
//      ->setSettings([
//        'max_length' => 50,
//        'text_processing' => 0,
//      ])
//      ->setDefaultValue('')
//      ->setDisplayOptions('view', [
//        'label' => 'above',
//        'type' => 'string',
//        'weight' => -4,
//      ])
//      ->setDisplayOptions('form', [
//        'type' => 'string_textfield',
//        'weight' => -4,
//      ])
//      ->setDisplayConfigurable('form', TRUE)
//      ->setDisplayConfigurable('view', TRUE);

    $fields['status'] = BaseFieldDefinition::create('boolean')
      ->setLabel(t('Publishing status'))
      ->setDescription(t('A boolean indicating whether the Volante is published.'))
      ->setDefaultValue(TRUE);

    $fields['created'] = BaseFieldDefinition::create('created')
      ->setLabel(t('Created'))
      ->setDescription(t('The time that the entity was created.'));

    $fields['changed'] = BaseFieldDefinition::create('changed')
      ->setLabel(t('Changed'))
      ->setDescription(t('The time that the entity was last edited.'));
    
    $fields['rivestimento'] = BaseFieldDefinition::create('string')
      ->setLabel(t('Rivestimento'))
      ->setDescription(t('Tipo di rivestimento del Volante (vera pelle, pelle sintetica, gomma...).'))
      ->setSettings([
        'max_length' => 50,
        'text_processing' => 0,
      ])
      ->setDefaultValue('')
      ->setDisplayOptions('view', [
        'label' => 'above',
        'type' => 'string',
        'weight' => -100,
      ])
      ->setDisplayOptions('form', [
        'type' => 'string_textfield',
        'weight' => -100,
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);
    
    $fields['auto'] = BaseFieldDefinition::create('entity_reference')
      ->setLabel(t('Auto'))
      ->setDescription(t('Auto cui il Volante appartiene.'))
      ->setRequired(TRUE)
      ->setSetting('target_type', 'auto')
      ->setSetting('handler', 'default')
      ->setTranslatable(TRUE)
      ->setDisplayOptions('view', [
        'label' => 'hidden',
        'weight' => 100,
      ])
      ->setDisplayOptions('form', [
        'type' => 'entity_reference_autocomplete',
        'weight' => 100,
        'settings' => [
          'match_operator' => 'CONTAINS',
          'size' => '60',
          'autocomplete_type' => 'tags',
          'placeholder' => '',
        ],
      ])
      ->setDisplayConfigurable('form', TRUE)
      ->setDisplayConfigurable('view', TRUE);

    return $fields;
  }

}
