<?php

namespace Drupal\auto;

use Drupal\Core\Entity\EntityAccessControlHandler;
use Drupal\Core\Entity\EntityInterface;
use Drupal\Core\Session\AccountInterface;
use Drupal\Core\Access\AccessResult;

/**
 * Access controller for the Ruota entity.
 *
 * @see \Drupal\auto\Entity\Ruota.
 */
class RuotaAccessControlHandler extends EntityAccessControlHandler {

  /**
   * {@inheritdoc}
   */
  protected function checkAccess(EntityInterface $entity, $operation, AccountInterface $account) {
    /** @var \Drupal\auto\Entity\RuotaInterface $entity */
    switch ($operation) {
      case 'view':
        if (!$entity->isPublished()) {
          return AccessResult::allowedIfHasPermission($account, 'view unpublished ruota entities');
        }
        return AccessResult::allowedIfHasPermission($account, 'view published ruota entities');

      case 'update':
        return AccessResult::allowedIfHasPermission($account, 'edit ruota entities');

      case 'delete':
        return AccessResult::allowedIfHasPermission($account, 'delete ruota entities');
    }

    // Unknown operation, no opinion.
    return AccessResult::neutral();
  }

  /**
   * {@inheritdoc}
   */
  protected function checkCreateAccess(AccountInterface $account, array $context, $entity_bundle = NULL) {
    return AccessResult::allowedIfHasPermission($account, 'add ruota entities');
  }

}
